package com.nisum.offertransactionservice.controller;

import com.github.tomakehurst.wiremock.junit.WireMockRule;
import com.nisum.offertransactionservice.dto.*;
import com.nisum.offertransactionservice.model.OfferLookup;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.cloud.contract.wiremock.AutoConfigureWireMock;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static com.github.tomakehurst.wiremock.client.WireMock.*;


@SpringBootTest(webEnvironment= SpringBootTest.WebEnvironment.DEFINED_PORT)
@AutoConfigureWireMock
@ActiveProfiles(value = "integration")
public class OfferTransactionControllerIntegrationTest extends AbstractTestBase{

    @Rule
    public WireMockRule wireMockRule = new WireMockRule(7072);

    @Autowired
    private TestRestTemplate testRestTemplate;


    @Before
    public void setUp() {
        mockForOfferTransactionCall();
        mockForEndOfSale();
        mockForEndOfTransaction();
    }

    private void mockForOfferTransactionCall() {
        mWireMockServer.stubFor(post(urlEqualTo("/offerTransactionCall"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", "application/json")
                        .withBodyFile("response.json")
                ));
    }

    private void mockForEndOfSale() {
        mWireMockServer.stubFor(post(urlEqualTo("/endOfSale"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", "application/json")
                        .withBodyFile("endOfSale-response.json")
                ));
    }

    private void mockForEndOfTransaction() {
        mWireMockServer.stubFor(post(urlEqualTo("/endOfTransaction"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", "application/json")
                        .withBodyFile("endOfTransaction-response.json")
                ));
    }

    @Test
    public void getOfferTransactionResponseTest() {
        URI uri = URI.create("http://localhost:7072/offerTransactionCall");
        RequestEntity requestEntity = new RequestEntity(getOfferTransactionRequest(),HttpMethod.POST,uri);

        ResponseEntity<OfferTransactionResponse> responseEntity = testRestTemplate.exchange("http://localhost:7072/offerTransactionCall", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<OfferTransactionResponse>(){}
        );
        OfferTransactionResponse offerTransactionResponse = responseEntity.getBody();
        System.out.println(offerTransactionResponse);

    }

    @Test
    public void endOfSaleTest() {
        URI uri = URI.create("http://localhost:7072/endOfSale");
        RequestEntity requestEntity = new RequestEntity(getEndOfSaleReq(),HttpMethod.POST,uri);

        ResponseEntity<EndOfSaleResponse> responseEntity = testRestTemplate.exchange("http://localhost:7072/endOfSale", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<EndOfSaleResponse>(){}
        );
        EndOfSaleResponse endOfSaleResponse = responseEntity.getBody();
        Assert.assertNotNull(endOfSaleResponse.getUuId());

    }

    @Test
    public void endOfTransactionTest() {
        URI uri = URI.create("http://localhost:7072/endOfTransaction");
        String uuid = "86fd4146-0540-405b-b621-a95f4ccdfa0d";
        RequestEntity requestEntity = new RequestEntity(uuid,HttpMethod.POST,uri);

        ResponseEntity<String> responseEntity = testRestTemplate.exchange("http://localhost:7072/endOfTransaction", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<String>(){}
        );
        String endOfTransactionRes = responseEntity.getBody();
        Assert.assertNotNull(endOfTransactionRes);

    }

    private EndOfSaleReq getEndOfSaleReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransRes());
        endOfSaleReq.setOffers(returnOffers());
        return endOfSaleReq;
    }


    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        offerTransactionResponse.setTransactionId("123456788899999998");
        return offerTransactionResponse;
    }



    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup(1L, "12", 23L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup(2L, "13", 24L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId(54321L);
        offerTransactionRequest.setItemList(getItemList());
        return offerTransactionRequest;
    }

    private List<Item> getItemList() {
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        return itemList;
    }
}
